package com.sun.media.sound;

import java.util.Arrays;

public final class SoftReverb implements SoftAudioProcessor
{
    private float roomsize;
    private float damp;
    private float gain;
    private SoftReverb.Delay delay;
    private SoftReverb.Comb[] combL;
    private SoftReverb.Comb[] combR;
    private SoftReverb.AllPass[] allpassL;
    private SoftReverb.AllPass[] allpassR;
    private float[] input;
    private float[] out;
    private float[] pre1;
    private float[] pre2;
    private float[] pre3;
    private boolean denormal_flip;
    private boolean mix;
    private SoftAudioBuffer inputA;
    private SoftAudioBuffer left;
    private SoftAudioBuffer right;
    private boolean dirty;
    private float dirty_roomsize;
    private float dirty_damp;
    private float dirty_predelay;
    private float dirty_gain;
    private float samplerate;
    private boolean light;
    private boolean silent;
    
    public SoftReverb() {
        this.gain = 1.0f;
        this.denormal_flip = false;
        this.mix = true;
        this.dirty = true;
        this.light = true;
        this.silent = true;
    }
    
    @Override
    public void init(float samplerate, float n) {
        this.samplerate = samplerate;
        double n2 = (double)samplerate / 44100.0;
        int n3 = 23;
        this.delay = new SoftReverb.Delay();
        this.combL = new SoftReverb.Comb[8];
        this.combR = new SoftReverb.Comb[8];
        this.combL[0] = new SoftReverb.Comb((int)(n2 * 1116.0));
        this.combR[0] = new SoftReverb.Comb((int)(n2 * (double)(1116 + n3)));
        this.combL[1] = new SoftReverb.Comb((int)(n2 * 1188.0));
        this.combR[1] = new SoftReverb.Comb((int)(n2 * (double)(1188 + n3)));
        this.combL[2] = new SoftReverb.Comb((int)(n2 * 1277.0));
        this.combR[2] = new SoftReverb.Comb((int)(n2 * (double)(1277 + n3)));
        this.combL[3] = new SoftReverb.Comb((int)(n2 * 1356.0));
        this.combR[3] = new SoftReverb.Comb((int)(n2 * (double)(1356 + n3)));
        this.combL[4] = new SoftReverb.Comb((int)(n2 * 1422.0));
        this.combR[4] = new SoftReverb.Comb((int)(n2 * (double)(1422 + n3)));
        this.combL[5] = new SoftReverb.Comb((int)(n2 * 1491.0));
        this.combR[5] = new SoftReverb.Comb((int)(n2 * (double)(1491 + n3)));
        this.combL[6] = new SoftReverb.Comb((int)(n2 * 1557.0));
        this.combR[6] = new SoftReverb.Comb((int)(n2 * (double)(1557 + n3)));
        this.combL[7] = new SoftReverb.Comb((int)(n2 * 1617.0));
        this.combR[7] = new SoftReverb.Comb((int)(n2 * (double)(1617 + n3)));
        this.allpassL = new SoftReverb.AllPass[4];
        this.allpassR = new SoftReverb.AllPass[4];
        this.allpassL[0] = new SoftReverb.AllPass((int)(n2 * 556.0));
        this.allpassR[0] = new SoftReverb.AllPass((int)(n2 * (double)(556 + n3)));
        this.allpassL[1] = new SoftReverb.AllPass((int)(n2 * 441.0));
        this.allpassR[1] = new SoftReverb.AllPass((int)(n2 * (double)(441 + n3)));
        this.allpassL[2] = new SoftReverb.AllPass((int)(n2 * 341.0));
        this.allpassR[2] = new SoftReverb.AllPass((int)(n2 * (double)(341 + n3)));
        this.allpassL[3] = new SoftReverb.AllPass((int)(n2 * 225.0));
        this.allpassR[3] = new SoftReverb.AllPass((int)(n2 * (double)(225 + n3)));
        for (int i = 0; i < this.allpassL.length; ++i) {
            this.allpassL[i].setFeedBack(0.5f);
            this.allpassR[i].setFeedBack(0.5f);
        }
        this.globalParameterControlChange(new int[] { 129 }, 0L, 4L);
    }
    
    @Override
    public void setInput(int n, SoftAudioBuffer inputA) {
        if (n == 0) {
            this.inputA = inputA;
        }
    }
    
    @Override
    public void setOutput(int n, SoftAudioBuffer softAudioBuffer) {
        if (n == 0) {
            this.left = softAudioBuffer;
        }
        if (n == 1) {
            this.right = softAudioBuffer;
        }
    }
    
    @Override
    public void setMixMode(boolean mix) {
        this.mix = mix;
    }
    
    @Override
    public void processAudio() {
        int silent = this.inputA.isSilent();
        if (silent == 0) {
            this.silent = false;
        }
        if (this.silent) {
            if (!this.mix) {
                this.left.clear();
                this.right.clear();
            }
            return;
        }
        else {
            float[] array = this.inputA.array();
            float[] array2 = this.left.array();
            float[] array3 = (this.right == null) ? null : this.right.array();
            int length = array.length;
            if (this.input == null || this.input.length < length) {
                this.input = new float[length];
            }
            float n = this.gain * 0.018f / 2.0f;
            this.denormal_flip = !this.denormal_flip;
            if (this.denormal_flip) {
                for (int j = 0; j < length; ++j) {
                    this.input[j] = array[j] * n + 1.0E-20f;
                }
            }
            else {
                for (int i = 0; i < length; ++i) {
                    this.input[i] = array[i] * n - 1.0E-20f;
                }
            }
            this.delay.processReplace(this.input);
            if (this.light && array3 != null) {
                if (this.pre1 == null || this.pre1.length < length) {
                    this.pre1 = new float[length];
                    this.pre2 = new float[length];
                    this.pre3 = new float[length];
                }
                for (int n4 = 0; n4 < this.allpassL.length; ++n4) {
                    this.allpassL[n4].processReplace(this.input);
                }
                this.combL[0].processReplace(this.input, this.pre3);
                this.combL[1].processReplace(this.input, this.pre3);
                this.combL[2].processReplace(this.input, this.pre1);
                for (int n5 = 4; n5 < this.combL.length - 2; n5 += 2) {
                    this.combL[n5].processMix(this.input, this.pre1);
                }
                this.combL[3].processReplace(this.input, this.pre2);
                for (int n6 = 5; n6 < this.combL.length - 2; n6 += 2) {
                    this.combL[n6].processMix(this.input, this.pre2);
                }
                if (!this.mix) {
                    Arrays.fill(array3, 0.0f);
                    Arrays.fill(array2, 0.0f);
                }
                for (int n7 = this.combR.length - 2; n7 < this.combR.length; ++n7) {
                    this.combR[n7].processMix(this.input, array3);
                }
                for (int n8 = this.combL.length - 2; n8 < this.combL.length; ++n8) {
                    this.combL[n8].processMix(this.input, array2);
                }
                for (int n9 = 0; n9 < length; ++n9) {
                    float n10 = this.pre1[n9] - this.pre2[n9];
                    float n11 = this.pre3[n9];
                    int n12 = n9;
                    array2[n12] += n11 + n10;
                    int n13 = n9;
                    array3[n13] += n11 - n10;
                }
            }
            else {
                if (this.out == null || this.out.length < length) {
                    this.out = new float[length];
                }
                if (array3 != null) {
                    if (!this.mix) {
                        Arrays.fill(array3, 0.0f);
                    }
                    this.allpassR[0].processReplace(this.input, this.out);
                    for (int k = 1; k < this.allpassR.length; ++k) {
                        this.allpassR[k].processReplace(this.out);
                    }
                    for (int l = 0; l < this.combR.length; ++l) {
                        this.combR[l].processMix(this.out, array3);
                    }
                }
                if (!this.mix) {
                    Arrays.fill(array2, 0.0f);
                }
                this.allpassL[0].processReplace(this.input, this.out);
                for (int n2 = 1; n2 < this.allpassL.length; ++n2) {
                    this.allpassL[n2].processReplace(this.out);
                }
                for (int n3 = 0; n3 < this.combL.length; ++n3) {
                    this.combL[n3].processMix(this.out, array2);
                }
            }
            if (!(silent == 0)) {
                this.silent = true;
                int n14 = 0;
                while (n14 < length) {
                    float n15 = array2[n14];
                    if ((double)n15 > 1.0E-10 || (double)n15 < -1.0E-10) {
                        this.silent = false;
                        break;
                    }
                    ++n14;
                }
            }
            return;
        }
    }
    
    @Override
    public void globalParameterControlChange(int[] array, long n, long n2) {
        if (array.length == 1 && array[0] == 129) {
            if (n == 0L) {
                if (n2 == 0L) {
                    this.dirty_roomsize = 1.1f;
                    this.dirty_damp = 5000.0f;
                    this.dirty_predelay = 0.0f;
                    this.dirty_gain = 4.0f;
                    this.dirty = true;
                }
                if (n2 == 1L) {
                    this.dirty_roomsize = 1.3f;
                    this.dirty_damp = 5000.0f;
                    this.dirty_predelay = 0.0f;
                    this.dirty_gain = 3.0f;
                    this.dirty = true;
                }
                if (n2 == 2L) {
                    this.dirty_roomsize = 1.5f;
                    this.dirty_damp = 5000.0f;
                    this.dirty_predelay = 0.0f;
                    this.dirty_gain = 2.0f;
                    this.dirty = true;
                }
                if (n2 == 3L) {
                    this.dirty_roomsize = 1.8f;
                    this.dirty_damp = 24000.0f;
                    this.dirty_predelay = 0.02f;
                    this.dirty_gain = 1.5f;
                    this.dirty = true;
                }
                if (n2 == 4L) {
                    this.dirty_roomsize = 1.8f;
                    this.dirty_damp = 24000.0f;
                    this.dirty_predelay = 0.03f;
                    this.dirty_gain = 1.5f;
                    this.dirty = true;
                }
                if (n2 == 8L) {
                    this.dirty_roomsize = 1.3f;
                    this.dirty_damp = 2500.0f;
                    this.dirty_predelay = 0.0f;
                    this.dirty_gain = 6.0f;
                    this.dirty = true;
                }
            }
            else if (n == 1L) {
                this.dirty_roomsize = (float)Math.exp((double)(n2 - 40L) * 0.025);
                this.dirty = true;
            }
        }
    }
    
    @Override
    public void processControlLogic() {
        if (this.dirty) {
            this.dirty = false;
            this.setRoomSize(this.dirty_roomsize);
            this.setDamp(this.dirty_damp);
            this.setPreDelay(this.dirty_predelay);
            this.setGain(this.dirty_gain);
        }
    }
    
    public void setRoomSize(float n) {
        this.roomsize = 1.0f - 0.17f / n;
        for (int i = 0; i < this.combL.length; ++i) {
            this.combL[i].feedback = this.roomsize;
            this.combR[i].feedback = this.roomsize;
        }
    }
    
    public void setPreDelay(float n) {
        this.delay.setDelay((int)(n * this.samplerate));
    }
    
    public void setGain(float gain) {
        this.gain = gain;
    }
    
    public void setDamp(float n) {
        double n2 = 2.0 - Math.cos((double)(n / this.samplerate) * 6.283185307179586);
        this.damp = (float)(n2 - Math.sqrt(n2 * n2 - 1.0));
        if (this.damp > 1.0f) {
            this.damp = 1.0f;
        }
        if (this.damp < 0.0f) {
            this.damp = 0.0f;
        }
        for (int i = 0; i < this.combL.length; ++i) {
            this.combL[i].setDamp(this.damp);
            this.combR[i].setDamp(this.damp);
        }
    }
    
    public void setLightMode(boolean light) {
        this.light = light;
    }
    
    private static final class Delay
    {
        private float[] delaybuffer;
        private int rovepos;
        
        Delay() {
            this.rovepos = 0;
            this.delaybuffer = null;
        }
        
        public void setDelay(int n) {
            if (n == 0) {
                this.delaybuffer = null;
            }
            else {
                this.delaybuffer = new float[n];
            }
            this.rovepos = 0;
        }
        
        public void processReplace(float[] array) {
            if (this.delaybuffer == null) {
                return;
            }
            int length = array.length;
            int length2 = this.delaybuffer.length;
            int rovepos = this.rovepos;
            for (int i = 0; i < length; ++i) {
                float n = array[i];
                array[i] = this.delaybuffer[rovepos];
                this.delaybuffer[rovepos] = n;
                if (++rovepos == length2) {
                    rovepos = 0;
                }
            }
            this.rovepos = rovepos;
            return;
        }
    }
    
    private static final class AllPass
    {
        private final float[] delaybuffer;
        private final int delaybuffersize;
        private int rovepos;
        private float feedback;
        
        AllPass(int delaybuffersize) {
            this.rovepos = 0;
            this.delaybuffer = new float[delaybuffersize];
            this.delaybuffersize = delaybuffersize;
        }
        
        public void setFeedBack(float feedback) {
            this.feedback = feedback;
        }
        
        public void processReplace(float[] array) {
            int length = array.length;
            int delaybuffersize = this.delaybuffersize;
            int rovepos = this.rovepos;
            for (int i = 0; i < length; ++i) {
                float n = this.delaybuffer[rovepos];
                float n2 = array[i];
                array[i] = n - n2;
                this.delaybuffer[rovepos] = n2 + n * this.feedback;
                if (++rovepos == delaybuffersize) {
                    rovepos = 0;
                }
            }
            this.rovepos = rovepos;
        }
        
        public void processReplace(float[] array, float[] array2) {
            int length = array.length;
            int delaybuffersize = this.delaybuffersize;
            int rovepos = this.rovepos;
            for (int i = 0; i < length; ++i) {
                float n = this.delaybuffer[rovepos];
                float n2 = array[i];
                array2[i] = n - n2;
                this.delaybuffer[rovepos] = n2 + n * this.feedback;
                if (++rovepos == delaybuffersize) {
                    rovepos = 0;
                }
            }
            this.rovepos = rovepos;
        }
    }
    
    private static final class Comb
    {
        private final float[] delaybuffer;
        private final int delaybuffersize;
        private int rovepos;
        private float feedback;
        private float filtertemp;
        private float filtercoeff1;
        private float filtercoeff2;
        
        Comb(int delaybuffersize) {
            this.rovepos = 0;
            this.filtertemp = 0.0f;
            this.filtercoeff1 = 0.0f;
            this.filtercoeff2 = 1.0f;
            this.delaybuffer = new float[delaybuffersize];
            this.delaybuffersize = delaybuffersize;
        }
        
        public void setFeedBack(float feedback) {
            this.feedback = feedback;
            this.filtercoeff2 = (1.0f - this.filtercoeff1) * feedback;
        }
        
        public void processMix(float[] array, float[] array2) {
            int length = array.length;
            int delaybuffersize = this.delaybuffersize;
            int rovepos = this.rovepos;
            float filtertemp = this.filtertemp;
            float filtercoeff1 = this.filtercoeff1;
            float filtercoeff2 = this.filtercoeff2;
            for (int i = 0; i < length; ++i) {
                float n = this.delaybuffer[rovepos];
                filtertemp = n * filtercoeff2 + filtertemp * filtercoeff1;
                int n2 = i;
                array2[n2] += n;
                this.delaybuffer[rovepos] = array[i] + filtertemp;
                if (++rovepos == delaybuffersize) {
                    rovepos = 0;
                }
            }
            this.filtertemp = filtertemp;
            this.rovepos = rovepos;
        }
        
        public void processReplace(float[] array, float[] array2) {
            int length = array.length;
            int delaybuffersize = this.delaybuffersize;
            int rovepos = this.rovepos;
            float filtertemp = this.filtertemp;
            float filtercoeff1 = this.filtercoeff1;
            float filtercoeff2 = this.filtercoeff2;
            for (int i = 0; i < length; ++i) {
                float n = this.delaybuffer[rovepos];
                filtertemp = n * filtercoeff2 + filtertemp * filtercoeff1;
                array2[i] = n;
                this.delaybuffer[rovepos] = array[i] + filtertemp;
                if (++rovepos == delaybuffersize) {
                    rovepos = 0;
                }
            }
            this.filtertemp = filtertemp;
            this.rovepos = rovepos;
        }
        
        public void setDamp(float filtercoeff1) {
            this.filtercoeff1 = filtercoeff1;
            this.filtercoeff2 = (1.0f - this.filtercoeff1) * this.feedback;
        }
    }
}
